@@ -1,11 +1,12 @@
 use std::fs::{File, OpenOptions};
+use std::io::Write;
 use std::mem::MaybeUninit;
 use std::os::unix::io::{AsRawFd, RawFd};
 use std::path::Path;
 use std::{io, mem};
 
 use crate::constants::*;
-use crate::{nix_err, sys, AttributeSet, AttributeSetRef, InputEvent, InputId, Key};
+use crate::{sys, AttributeSet, AttributeSetRef, InputEvent, InputId, Key};
 
 fn ioctl_get_cstring(
     f: unsafe fn(RawFd, &mut [u8]) -> nix::Result<libc::c_int>,
@@ -38,6 +39,8 @@ const ABSINFO_ZERO: libc::input_absinfo = libc::input_absinfo {
 pub(crate) const ABS_VALS_INIT: [libc::input_absinfo; AbsoluteAxisType::COUNT] =
     [ABSINFO_ZERO; AbsoluteAxisType::COUNT];
 
+const INPUT_KEYMAP_BY_INDEX: u8 = 1;
+
 /// A physical or virtual device supported by evdev.
 ///
 /// Each device corresponds to a path typically found in `/dev/input`, and supports access via
@@ -59,11 +62,19 @@ pub struct RawDevice {
     supported_switch: Option<AttributeSet<SwitchType>>,
     supported_led: Option<AttributeSet<LedType>>,
     supported_misc: Option<AttributeSet<MiscType>>,
+    auto_repeat: Option<AutoRepeat>,
     // ff: Option<AttributeSet<_>>,
     // ff_stat: Option<FFStatus>,
-    // rep: Option<Repeat>,
     supported_snd: Option<AttributeSet<SoundType>>,
     pub(crate) event_buf: Vec<libc::input_event>,
+    grabbed: bool,
+}
+
+#[derive(Debug, Clone)]
+#[repr(C)]
+pub struct AutoRepeat {
+    pub delay: u32,
+    pub period: u32,
 }
 
 impl RawDevice {
@@ -87,9 +98,7 @@ impl RawDevice {
 
         let ty = {
             let mut ty = AttributeSet::<EventType>::new();
-            unsafe {
-                sys::eviocgbit_type(file.as_raw_fd(), ty.as_mut_raw_slice()).map_err(nix_err)?
-            };
+            unsafe { sys::eviocgbit_type(file.as_raw_fd(), ty.as_mut_raw_slice())? };
             ty
         };
 
@@ -102,12 +111,12 @@ impl RawDevice {
 
         let id = unsafe {
             let mut id = MaybeUninit::uninit();
-            sys::eviocgid(file.as_raw_fd(), id.as_mut_ptr()).map_err(nix_err)?;
+            sys::eviocgid(file.as_raw_fd(), id.as_mut_ptr())?;
             id.assume_init()
         };
         let mut driver_version: i32 = 0;
         unsafe {
-            sys::eviocgversion(file.as_raw_fd(), &mut driver_version).map_err(nix_err)?;
+            sys::eviocgversion(file.as_raw_fd(), &mut driver_version)?;
         }
         let driver_version = (
             ((driver_version >> 16) & 0xff) as u8,
@@ -117,17 +126,13 @@ impl RawDevice {
 
         let props = {
             let mut props = AttributeSet::<PropType>::new();
-            unsafe {
-                sys::eviocgprop(file.as_raw_fd(), props.as_mut_raw_slice()).map_err(nix_err)?
-            };
+            unsafe { sys::eviocgprop(file.as_raw_fd(), props.as_mut_raw_slice())? };
             props
         }; // FIXME: handle old kernel
 
         let supported_keys = if ty.contains(EventType::KEY) {
             let mut keys = AttributeSet::<Key>::new();
-            unsafe {
-                sys::eviocgbit_key(file.as_raw_fd(), keys.as_mut_raw_slice()).map_err(nix_err)?
-            };
+            unsafe { sys::eviocgbit_key(file.as_raw_fd(), keys.as_mut_raw_slice())? };
             Some(keys)
         } else {
             None
@@ -135,10 +140,7 @@ impl RawDevice {
 
         let supported_relative = if ty.contains(EventType::RELATIVE) {
             let mut rel = AttributeSet::<RelativeAxisType>::new();
-            unsafe {
-                sys::eviocgbit_relative(file.as_raw_fd(), rel.as_mut_raw_slice())
-                    .map_err(nix_err)?
-            };
+            unsafe { sys::eviocgbit_relative(file.as_raw_fd(), rel.as_mut_raw_slice())? };
             Some(rel)
         } else {
             None
@@ -146,10 +148,7 @@ impl RawDevice {
 
         let supported_absolute = if ty.contains(EventType::ABSOLUTE) {
             let mut abs = AttributeSet::<AbsoluteAxisType>::new();
-            unsafe {
-                sys::eviocgbit_absolute(file.as_raw_fd(), abs.as_mut_raw_slice())
-                    .map_err(nix_err)?
-            };
+            unsafe { sys::eviocgbit_absolute(file.as_raw_fd(), abs.as_mut_raw_slice())? };
             Some(abs)
         } else {
             None
@@ -157,10 +156,7 @@ impl RawDevice {
 
         let supported_switch = if ty.contains(EventType::SWITCH) {
             let mut switch = AttributeSet::<SwitchType>::new();
-            unsafe {
-                sys::eviocgbit_switch(file.as_raw_fd(), switch.as_mut_raw_slice())
-                    .map_err(nix_err)?
-            };
+            unsafe { sys::eviocgbit_switch(file.as_raw_fd(), switch.as_mut_raw_slice())? };
             Some(switch)
         } else {
             None
@@ -168,9 +164,7 @@ impl RawDevice {
 
         let supported_led = if ty.contains(EventType::LED) {
             let mut led = AttributeSet::<LedType>::new();
-            unsafe {
-                sys::eviocgbit_led(file.as_raw_fd(), led.as_mut_raw_slice()).map_err(nix_err)?
-            };
+            unsafe { sys::eviocgbit_led(file.as_raw_fd(), led.as_mut_raw_slice())? };
             Some(led)
         } else {
             None
@@ -178,9 +172,7 @@ impl RawDevice {
 
         let supported_misc = if ty.contains(EventType::MISC) {
             let mut misc = AttributeSet::<MiscType>::new();
-            unsafe {
-                sys::eviocgbit_misc(file.as_raw_fd(), misc.as_mut_raw_slice()).map_err(nix_err)?
-            };
+            unsafe { sys::eviocgbit_misc(file.as_raw_fd(), misc.as_mut_raw_slice())? };
             Some(misc)
         } else {
             None
@@ -190,14 +182,30 @@ impl RawDevice {
 
         let supported_snd = if ty.contains(EventType::SOUND) {
             let mut snd = AttributeSet::<SoundType>::new();
-            unsafe {
-                sys::eviocgbit_sound(file.as_raw_fd(), snd.as_mut_raw_slice()).map_err(nix_err)?
-            };
+            unsafe { sys::eviocgbit_sound(file.as_raw_fd(), snd.as_mut_raw_slice())? };
             Some(snd)
         } else {
             None
         };
 
+        let auto_repeat = if ty.contains(EventType::REPEAT) {
+            let mut auto_repeat: AutoRepeat = AutoRepeat {
+                delay: 0,
+                period: 0,
+            };
+
+            unsafe {
+                sys::eviocgrep(
+                    file.as_raw_fd(),
+                    &mut auto_repeat as *mut AutoRepeat as *mut [u32; 2],
+                )?;
+            }
+
+            Some(auto_repeat)
+        } else {
+            None
+        };
+
         Ok(RawDevice {
             file,
             ty,
@@ -214,7 +222,9 @@ impl RawDevice {
             supported_led,
             supported_misc,
             supported_snd,
+            auto_repeat,
             event_buf: Vec::new(),
+            grabbed: false,
         })
     }
 
@@ -238,6 +248,11 @@ impl RawDevice {
         InputId::from(self.id)
     }
 
+    /// Returns the current auto repeat settings
+    pub fn get_auto_repeat(&self) -> Option<AutoRepeat> {
+        self.auto_repeat.clone()
+    }
+
     /// Returns the set of supported "properties" for the device (see `INPUT_PROP_*` in kernel headers)
     pub fn properties(&self) -> &AttributeSetRef<PropType> {
         &self.props
@@ -360,10 +375,6 @@ impl RawDevice {
         self.supported_misc.as_deref()
     }
 
-    // pub fn supported_repeats(&self) -> Option<Repeat> {
-    //     self.rep
-    // }
-
     /// Returns the set of supported simple sounds supported by a device.
     ///
     /// You can use these to make really annoying beep sounds come from an internal self-test
@@ -386,7 +397,7 @@ impl RawDevice {
 
         // use libc::read instead of nix::unistd::read b/c we need to pass an uninitialized buf
         let res = unsafe { libc::read(fd, spare_capacity.as_mut_ptr() as _, spare_capacity_size) };
-        let bytes_read = nix::errno::Errno::result(res).map_err(nix_err)?;
+        let bytes_read = nix::errno::Errno::result(res)?;
         let num_read = bytes_read as usize / mem::size_of::<libc::input_event>();
         unsafe {
             let len = self.event_buf.len();
@@ -442,9 +453,8 @@ impl RawDevice {
     /// [`get_key_state`](Self::get_key_state) instead.
     #[inline]
     pub fn update_key_state(&self, key_vals: &mut AttributeSet<Key>) -> io::Result<()> {
-        unsafe { sys::eviocgkey(self.as_raw_fd(), key_vals.as_mut_raw_slice()) }
-            .map(|_| ())
-            .map_err(nix_err)
+        unsafe { sys::eviocgkey(self.as_raw_fd(), key_vals.as_mut_raw_slice())? };
+        Ok(())
     }
 
     /// Fetch the current kernel absolute axis state directly into the provided buffer.
@@ -462,8 +472,7 @@ impl RawDevice {
                 // handling later removed. not sure what the intention of "handling that later" was
                 // the abs data seems to be fine (tested ABS_MT_POSITION_X/Y)
                 unsafe {
-                    sys::eviocgabs(self.as_raw_fd(), idx as u32, &mut abs_vals[idx as usize])
-                        .map_err(nix_err)?
+                    sys::eviocgabs(self.as_raw_fd(), idx as u32, &mut abs_vals[idx as usize])?
                 };
             }
         }
@@ -478,9 +487,8 @@ impl RawDevice {
         &self,
         switch_vals: &mut AttributeSet<SwitchType>,
     ) -> io::Result<()> {
-        unsafe { sys::eviocgsw(self.as_raw_fd(), switch_vals.as_mut_raw_slice()) }
-            .map(|_| ())
-            .map_err(nix_err)
+        unsafe { sys::eviocgsw(self.as_raw_fd(), switch_vals.as_mut_raw_slice())? };
+        Ok(())
     }
 
     /// Fetch the current kernel LED state directly into the provided buffer.
@@ -488,9 +496,94 @@ impl RawDevice {
     /// [`get_led_state`](Self::get_led_state) instead.
     #[inline]
     pub fn update_led_state(&self, led_vals: &mut AttributeSet<LedType>) -> io::Result<()> {
-        unsafe { sys::eviocgled(self.as_raw_fd(), led_vals.as_mut_raw_slice()) }
-            .map(|_| ())
-            .map_err(nix_err)
+        unsafe { sys::eviocgled(self.as_raw_fd(), led_vals.as_mut_raw_slice())? };
+        Ok(())
+    }
+
+    /// Update the auto repeat delays
+    #[inline]
+    pub fn update_auto_repeat(&mut self, repeat: &AutoRepeat) -> io::Result<()> {
+        unsafe {
+            sys::eviocsrep(
+                self.as_raw_fd(),
+                repeat as *const AutoRepeat as *const [u32; 2],
+            )?;
+        }
+        self.auto_repeat = Some(repeat.clone());
+        Ok(())
+    }
+
+    /// Retrieve the scancode for a keycode, if any
+    pub fn get_scancode_by_keycode(&self, keycode: u32) -> io::Result<Vec<u8>> {
+        let mut keymap = libc::input_keymap_entry {
+            flags: 0,
+            len: 0,
+            index: 0,
+            keycode,
+            scancode: [0u8; 32],
+        };
+        unsafe { sys::eviocgkeycode_v2(self.as_raw_fd(), &mut keymap)? };
+        Ok(keymap.scancode[..keymap.len as usize].to_vec())
+    }
+
+    /// Retrieve the keycode and scancode by index, starting at 0
+    pub fn get_scancode_by_index(&self, index: u16) -> io::Result<(u32, Vec<u8>)> {
+        let mut keymap = libc::input_keymap_entry {
+            flags: INPUT_KEYMAP_BY_INDEX,
+            len: 0,
+            index,
+            keycode: 0,
+            scancode: [0u8; 32],
+        };
+
+        unsafe { sys::eviocgkeycode_v2(self.as_raw_fd(), &mut keymap)? };
+        Ok((
+            keymap.keycode,
+            keymap.scancode[..keymap.len as usize].to_vec(),
+        ))
+    }
+
+    /// Update a scancode by index. The return value is the previous keycode
+    pub fn update_scancode_by_index(
+        &self,
+        index: u16,
+        keycode: u32,
+        scancode: &[u8],
+    ) -> io::Result<u32> {
+        let len = scancode.len();
+
+        let mut keymap = libc::input_keymap_entry {
+            flags: INPUT_KEYMAP_BY_INDEX,
+            len: len as u8,
+            index,
+            keycode,
+            scancode: [0u8; 32],
+        };
+
+        keymap.scancode[..len].copy_from_slice(scancode);
+
+        let keycode = unsafe { sys::eviocskeycode_v2(self.as_raw_fd(), &keymap)? };
+
+        Ok(keycode as u32)
+    }
+
+    /// Update a scancode. The return value is the previous keycode
+    pub fn update_scancode(&self, keycode: u32, scancode: &[u8]) -> io::Result<u32> {
+        let len = scancode.len();
+
+        let mut keymap = libc::input_keymap_entry {
+            flags: 0,
+            len: len as u8,
+            index: 0,
+            keycode,
+            scancode: [0u8; 32],
+        };
+
+        keymap.scancode[..len].copy_from_slice(scancode);
+
+        let keycode = unsafe { sys::eviocskeycode_v2(self.as_raw_fd(), &keymap)? };
+
+        Ok(keycode as u32)
     }
 
     #[cfg(feature = "tokio")]
@@ -498,6 +591,37 @@ impl RawDevice {
     pub fn into_event_stream(self) -> io::Result<EventStream> {
         EventStream::new(self)
     }
+
+    pub fn grab(&mut self) -> io::Result<()> {
+        if !self.grabbed {
+            unsafe {
+                sys::eviocgrab(self.as_raw_fd(), 1)?;
+            }
+            self.grabbed = true;
+        }
+        Ok(())
+    }
+
+    pub fn ungrab(&mut self) -> io::Result<()> {
+        if self.grabbed {
+            unsafe {
+                sys::eviocgrab(self.as_raw_fd(), 0)?;
+            }
+            self.grabbed = false;
+        }
+        Ok(())
+    }
+
+    /// Send an event to the device.
+    ///
+    /// Events that are typically sent to devices are
+    /// [EventType::LED] (turn device LEDs on and off),
+    /// [EventType::SOUND] (play a sound on the device)
+    /// and [EventType::FORCEFEEDBACK] (play force feedback effects on the device, i.e. rumble).
+    pub fn send_events(&mut self, events: &[InputEvent]) -> io::Result<()> {
+        let bytes = unsafe { crate::cast_to_bytes(events) };
+        self.file.write_all(bytes)
+    }
 }
 
 impl AsRawFd for RawDevice {
@@ -576,8 +700,7 @@ mod tokio_stream {
     impl EventStream {
         pub(crate) fn new(device: RawDevice) -> io::Result<Self> {
             use nix::fcntl;
-            fcntl::fcntl(device.as_raw_fd(), fcntl::F_SETFL(fcntl::OFlag::O_NONBLOCK))
-                .map_err(nix_err)?;
+            fcntl::fcntl(device.as_raw_fd(), fcntl::F_SETFL(fcntl::OFlag::O_NONBLOCK))?;
             let device = AsyncFd::new(device)?;
             Ok(Self { device, index: 0 })
         }
