@@ -1,7 +1,7 @@
 use crate::constants::*;
 use crate::device_state::DeviceState;
 use crate::raw_stream::RawDevice;
-use crate::{AttributeSet, AttributeSetRef, InputEvent, InputEventKind, InputId, Key};
+use crate::{AttributeSet, AttributeSetRef, AutoRepeat, InputEvent, InputEventKind, InputId, Key};
 use std::os::unix::io::{AsRawFd, RawFd};
 use std::path::Path;
 use std::time::SystemTime;
@@ -85,6 +85,44 @@ impl Device {
         self.raw.input_id()
     }
 
+    /// Returns a struct containing the delay and period for auto repeat
+    pub fn get_auto_repeat(&self) -> Option<AutoRepeat> {
+        self.raw.get_auto_repeat()
+    }
+
+    /// Update the delay and period for autorepeat
+    pub fn update_auto_repeat(&mut self, repeat: &AutoRepeat) -> io::Result<()> {
+        self.raw.update_auto_repeat(repeat)
+    }
+
+    /// Retrieve the scancode for a keycode, if any
+    pub fn get_scancode_by_keycode(&self, keycode: Key) -> io::Result<Vec<u8>> {
+        self.raw.get_scancode_by_keycode(keycode.code() as u32)
+    }
+
+    /// Retrieve the keycode and scancode by index, starting at 0
+    pub fn get_scancode_by_index(&self, index: u16) -> io::Result<(u32, Vec<u8>)> {
+        self.raw.get_scancode_by_index(index)
+    }
+
+    /// Update a scancode. The return value is the previous keycode
+    pub fn update_scancode(&self, keycode: Key, scancode: &[u8]) -> io::Result<Key> {
+        self.raw
+            .update_scancode(keycode.code() as u32, scancode)
+            .map(|keycode| Key::new(keycode as u16))
+    }
+
+    /// Update a scancode by index. The return value is the previous keycode
+    pub fn update_scancode_by_index(
+        &self,
+        index: u16,
+        keycode: Key,
+        scancode: &[u8],
+    ) -> io::Result<u32> {
+        self.raw
+            .update_scancode_by_index(index, keycode.code() as u32, scancode)
+    }
+
     /// Returns the set of supported "properties" for the device (see `INPUT_PROP_*` in kernel headers)
     pub fn properties(&self) -> &AttributeSetRef<PropType> {
         self.raw.properties()
@@ -207,10 +245,6 @@ impl Device {
         self.raw.misc_properties()
     }
 
-    // pub fn supported_repeats(&self) -> Option<Repeat> {
-    //     self.rep
-    // }
-
     /// Returns the set of supported simple sounds supported by a device.
     ///
     /// You can use these to make really annoying beep sounds come from an internal self-test
@@ -295,6 +329,29 @@ impl Device {
     pub fn into_event_stream(self) -> io::Result<EventStream> {
         EventStream::new(self)
     }
+
+    /// Grab the device through a kernel syscall.
+    ///
+    /// This prevents other clients (including kernel-internal ones such as rfkill) from receiving
+    /// events from this device.
+    pub fn grab(&mut self) -> io::Result<()> {
+        self.raw.grab()
+    }
+
+    /// Ungrab the device through a kernel syscall.
+    pub fn ungrab(&mut self) -> io::Result<()> {
+        self.raw.ungrab()
+    }
+
+    /// Send an event to the device.
+    ///
+    /// Events that are typically sent to devices are
+    /// [EventType::LED] (turn device LEDs on and off),
+    /// [EventType::SOUND] (play a sound on the device)
+    /// and [EventType::FORCEFEEDBACK] (play force feedback effects on the device, i.e. rumble).
+    pub fn send_events(&mut self, events: &[InputEvent]) -> io::Result<()> {
+        self.raw.send_events(events)
+    }
 }
 
 impl AsRawFd for Device {
@@ -450,6 +507,7 @@ impl<'a> Iterator for FetchEventsSynced<'a> {
         // first: check if we need to emit compensatory events due to a SYN_DROPPED we found in the
         // last batch of blocks
         if let Some(ev) = compensate_events(&mut self.sync, &mut self.dev) {
+            self.dev.prev_state.process_event(ev);
             return Some(ev);
         }
         let state = &mut self.dev.state;
@@ -643,7 +701,6 @@ mod tokio_stream {
 
     use tokio_1 as tokio;
 
-    use crate::nix_err;
     use crate::raw_stream::poll_fn;
     use futures_core::{ready, Stream};
     use std::pin::Pin;
@@ -667,8 +724,7 @@ mod tokio_stream {
     impl EventStream {
         pub(crate) fn new(device: Device) -> io::Result<Self> {
             use nix::fcntl;
-            fcntl::fcntl(device.as_raw_fd(), fcntl::F_SETFL(fcntl::OFlag::O_NONBLOCK))
-                .map_err(nix_err)?;
+            fcntl::fcntl(device.as_raw_fd(), fcntl::F_SETFL(fcntl::OFlag::O_NONBLOCK))?;
             let device = AsyncFd::new(device)?;
             Ok(Self {
                 device,
